{
 Cinetraverse Config - GUI utility to change Cinetraverse settings
 Copyright (c) 2019 Sunkware
 PubKey FP: 6B6D C8E9 3438 6E9C 3D97  56E5 2CE9 A476 99EF 28F6

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.

 [ WWW: sunkware.org ]                         [ E-MAIL: sunkware@gmail.com ]
}

unit CinetraverseConfig;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls,
  Config;

type

  { TMainForm }

  TMainForm = class(TForm)
    btnSave: TButton;
    btnLoad: TButton;
    chkMute: TCheckBox;
    chkVSync: TCheckBox;
    cmbResolution: TComboBox;
    cmbLanguage: TComboBox;
    edtAppWinName: TEdit;
    gboxMode: TGroupBox;
    gboxVideo: TGroupBox;
    gboxAudio: TGroupBox;
    gboxInput: TGroupBox;
    gboxAdvVideo: TGroupBox;
    gboxLang: TGroupBox;
    gboxAdvInput: TGroupBox;
    lblMouseSensitivityValue: TLabel;
    lblFramerateValue: TLabel;
    lblMusicValue: TLabel;
    lblSpeechValue: TLabel;
    lblSoundValue: TLabel;
    lblMouseSensitivity: TLabel;
    lblFramerate: TLabel;
    lblAppWinName: TLabel;
    lblMusic: TLabel;
    lblSpeech: TLabel;
    lblSound: TLabel;
    lblResolution: TLabel;
    pages: TPageControl;
    pgBasic: TTabSheet;
    pgAdvanced: TTabSheet;
    rbRightHand: TRadioButton;
    rbLeftHand: TRadioButton;
    rbWindowed: TRadioButton;
    rbFullscreen: TRadioButton;
    rbFullscreenDesktop: TRadioButton;
    tbarSound: TTrackBar;
    tbarSpeech: TTrackBar;
    tbarMusic: TTrackBar;
    tbarFramerate: TTrackBar;
    tbarMouseSensitivity: TTrackBar;
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure chkMuteChange(Sender: TObject);
    procedure chkVSyncChange(Sender: TObject);
    procedure cmbLanguageChange(Sender: TObject);
    procedure cmbResolutionChange(Sender: TObject);
    procedure edtAppWinNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure rbFullscreenChange(Sender: TObject);
    procedure rbFullscreenDesktopChange(Sender: TObject);
    procedure rbLeftHandChange(Sender: TObject);
    procedure rbRightHandChange(Sender: TObject);
    procedure rbWindowedChange(Sender: TObject);
    procedure tbarFramerateChange(Sender: TObject);
    procedure tbarMouseSensitivityChange(Sender: TObject);
    procedure tbarMusicChange(Sender: TObject);
    procedure tbarSoundChange(Sender: TObject);
    procedure tbarSpeechChange(Sender: TObject);
  private

  public

  end;

const
  AppTitleBegin: String = 'Cinetraverse';
  AppVersion: String = '1.0.2';
  AppTitleEnd: String = 'Config';

var
  MainForm: TMainForm;

// procedure InitLists();
// procedure ApplyConfig();

implementation

{$R *.lfm}

{ TMainForm }

procedure InitLists();
var
  i: Integer;
begin
  // Resolutions
  MainForm.cmbResolution.Items.Append('Maximum');
  for i := 1 to Length(Resolutions)-1 do begin
    MainForm.cmbResolution.Items.Append(IntToStr(Resolutions[i][0]) + ' x ' + IntToStr(Resolutions[i][1]));
  end;
  MainForm.cmbResolution.ItemIndex := 0;

  // Languages
  for i := 0 to Length(Languages)-1 do begin
    MainForm.cmbLanguage.Items.Append(Languages[i][1]);
  end;
  MainForm.cmbLanguage.ItemIndex := 0;

end;

procedure ApplyConfig();
var
  ResStr: String;
  i, n: Integer;
  found: Boolean;
begin

  // App window name
  MainForm.edtAppWinName.Text := Cfg.AppWindowName;

  // Resolution
  found := false;
  n := MainForm.cmbResolution.Items.Count;
  i := 0;
  while (found = false) and (i < n) do begin
    if (Cfg.XRes = Resolutions[i][0]) and (Cfg.YRes = Resolutions[i][1]) then begin
      MainForm.cmbResolution.ItemIndex := i;
      found := true;
    end;
    i := i + 1;
  end;
  if (found = false) then begin // add loaded resolution to the end of the list and select it
    ResStr := IntToStr(Cfg.XRes) + ' x ' + IntToStr(Cfg.YRes);
    MainForm.cmbResolution.Items.Append(ResStr);
    MainForm.cmbResolution.ItemIndex := n;
  end;

  // Mode
  if (Cfg.FullScr = false) then MainForm.rbWindowed.Checked := true
    else if (Cfg.FullScrDesk = false) then MainForm.rbFullscreen.Checked := true
      else MainForm.rbFullscreenDesktop.Checked := true;

  // Vertical sync
  MainForm.chkVSync.Checked := Cfg.VSync;

  // Framerate
  MainForm.tbarFramerate.Position := Cfg.Framerate;

  // Mouse sensitivity
  MainForm.tbarMouseSensitivity.Position := Cfg.MouseSensitivity;

  // Audio volumes
  MainForm.tbarSound.Position := Cfg.VolSound;
  MainForm.tbarMusic.Position := Cfg.VolMusic;
  MainForm.tbarSpeech.Position := Cfg.VolSpeech;

  // Mute
  MainForm.chkMute.Checked := Cfg.Mute;

  // Left-hand / right-hand
  if (Cfg.LeftHand = true) then MainForm.rbLeftHand.Checked := true else MainForm.rbRightHand.Checked := true;

  // Language
  i := 0;
  found := false;
  while (found = false) and (i < Length(Languages)) do begin
    if (Languages[i][0] = Cfg.Lang) then begin
      MainForm.cmbLanguage.ItemIndex := i;
      found := true;
    end;
     i := i + 1;
  end;
  if (found = false) then begin
    MainForm.cmbLanguage.ItemIndex := 0;
  end;

end;

procedure TMainForm.btnSaveClick(Sender: TObject);
begin
  SaveConfig();
  MessageDlg('Info', 'Config saved.', mtInformation, [mbOk], 0, mbOk);
end;

procedure TMainForm.chkMuteChange(Sender: TObject);
begin
  Cfg.Mute := chkMute.Checked;
end;

procedure TMainForm.chkVSyncChange(Sender: TObject);
begin
  Cfg.VSync := chkVSync.Checked;
end;

procedure TMainForm.cmbLanguageChange(Sender: TObject);
begin
  Cfg.Lang := Languages[cmbLanguage.ItemIndex][0];
end;

procedure TMainForm.cmbResolutionChange(Sender: TObject);
var
  i: Integer;
  xyres: TStringArray;
begin
    i := cmbResolution.ItemIndex;
    if i = 0 then begin
      Cfg.XRes := 0;
      Cfg.YRes := 0;
    end else begin
      xyres := cmbResolution.Items[i].Split('x');

      Cfg.XRes := StrToInt(xyres[0].Trim());
      Cfg.YRes := StrToInt(xyres[1].Trim());
    end
end;

procedure TMainForm.edtAppWinNameChange(Sender: TObject);
begin
  Cfg.AppWindowName := edtAppWinName.Text;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MainForm.Caption := AppTitleBegin + ' ' + AppVersion + ' ' + AppTitleEnd;

  InitLists();
  LoadConfig();
  ApplyConfig();
end;

procedure TMainForm.rbFullscreenChange(Sender: TObject);
begin
  if (rbFullscreen.Checked = true) then begin
    Cfg.FullScr := true;
    Cfg.FullScrDesk := false;

    MainForm.cmbResolution.Enabled := true;
  end;
end;

procedure TMainForm.rbFullscreenDesktopChange(Sender: TObject);
begin
  if (rbFullscreenDesktop.Checked = true) then begin
    Cfg.FullScr := true;
    Cfg.FullScrDesk := true;

    MainForm.cmbResolution.ItemIndex := 0; // "maximum"
    Cfg.XRes := 0;
    Cfg.YRes := 0;
    MainForm.cmbResolution.Enabled := false;
  end;
end;

procedure TMainForm.rbLeftHandChange(Sender: TObject);
begin
  if (rbLeftHand.Checked = true) then Cfg.LeftHand := true;
end;

procedure TMainForm.rbRightHandChange(Sender: TObject);
begin
  if (rbRightHand.Checked = true) then Cfg.LeftHand := false;
end;

procedure TMainForm.rbWindowedChange(Sender: TObject);
begin
  if (rbWindowed.Checked = true) then begin
    Cfg.FullScr := false;
    Cfg.FullScrDesk := false;

    MainForm.cmbResolution.Enabled := true;
  end;
end;

procedure TMainForm.tbarFramerateChange(Sender: TObject);
begin
  Cfg.Framerate := tbarFramerate.Position;
  lblFramerateValue.Caption := IntToStr(Cfg.Framerate);
end;

procedure TMainForm.tbarMouseSensitivityChange(Sender: TObject);
begin
  Cfg.MouseSensitivity := tbarMouseSensitivity.Position;
  lblMouseSensitivityValue.Caption := IntToStr(Cfg.MouseSensitivity);
end;

procedure TMainForm.tbarSoundChange(Sender: TObject);
begin
  Cfg.VolSound := tbarSound.Position;
  lblSoundValue.Caption := IntToStr(Cfg.VolSound);
end;

procedure TMainForm.tbarMusicChange(Sender: TObject);
begin
  Cfg.VolMusic := tbarMusic.Position;
  lblMusicValue.Caption := IntToStr(Cfg.VolMusic);
end;

procedure TMainForm.tbarSpeechChange(Sender: TObject);
begin
  Cfg.VolSpeech := tbarSpeech.Position;
  lblSpeechValue.Caption := IntToStr(Cfg.VolSpeech);
end;

procedure TMainForm.btnLoadClick(Sender: TObject);
begin
  LoadConfig();
  ApplyConfig();
end;

end.

