{
 Cinetraverse Config - GUI utility to change Cinetraverse settings
 Copyright (c) 2019 Sunkware
 PubKey FP: 6B6D C8E9 3438 6E9C 3D97  56E5 2CE9 A476 99EF 28F6

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.

 [ WWW: sunkware.org ]                         [ E-MAIL: sunkware@gmail.com ]
}

unit Config;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  fpjson, jsonConf;

type

  TConfig = record
    AppWindowName: String;
    XRes: Integer;
    YRes: Integer;
    FullScr: Boolean;
    FullScrDesk: Boolean;
    VSync: Boolean;
    Framerate: Integer;
    MouseSensitivity: Integer;
    LeftHand: Boolean;
    Mute: Boolean;
    VolSound: Integer;
    VolMusic: Integer;
    VolSpeech: Integer;
    Lang: String;
  end;

const
  ConfigFilename: String = 'config.json';

  Resolutions: array[0..13, 0..1] of Integer = (
               (0, 0), // means "maximum resolution" (depends on window mode)
               (800, 450),
               (800, 600),
               (960, 540),
               (960, 720),
               (1024, 576),
               (1024, 600),
               (1024, 768),
               (1280, 720),
               (1280, 1024),
               (1366, 768),
               (1600, 900),
               (1600, 1200),
               (1920, 1080)
               );

  Languages: array[0..52, 0..1] of String = (
             ('afr', 'Afrikaans'),
             ('alb', 'Shqip'),
             ('amh', 'አማርኛ'),
             ('ara', 'العربية'),
             ('bas', 'Euskara'),
             ('ben', 'বাংলা'),
             ('bos', 'Bosanski'),
             ('cat', 'Català'),
             ('chi', '中文'),
             ('cro', 'Hrvatski'),
             ('cze', 'Čeština'),
             ('bul', 'Български'),
             ('dan', 'Dansk'),
             ('dut', 'Nederlands'),
             ('eng', 'English'),
             ('est', 'Eesti'),
             ('fil', 'Filipino'),
             ('fin', 'Suomi'),
             ('fre', 'Français'),
             ('gal', 'Galego'),
             ('ger', 'Deutsch'),
             ('gre', 'Ελληνικά'),
             ('heb', 'עברית'),
             ('hin', 'हिन्दी'),
             ('hun', 'Magyar'),
             ('ind', 'Bahasa Indonesia'),
             ('ice', 'Íslenska'),
             ('iri', 'Gaeilge'),
             ('ita', 'Italiano'),
             ('jap', '日本語'),
             ('kor', '한국어'),
             ('lat', 'Latviešu'),
             ('lit', 'Lietuvių'),
             ('mac', 'Македонски'),
             ('mal', 'മലയാളം'),
             ('nor', 'Norsk'),
             ('per', 'فارسی'),
             ('pol', 'Polski'),
             ('por', 'Português'),
             ('pun', 'ਪੰਜਾਬੀ'),
             ('rom', 'Română'),
             ('rus', 'Русский'),
             ('ser', 'Српски'),
             ('slo', 'Slovenčina'),
             ('spa', 'Español'),
             ('swa', 'Kiswahili'),
             ('swe', 'Svenska'),
             ('tam', 'தமிழ்'),
             ('tha', 'ภาษาไทย'),
             ('tur', 'Türkçe'),
             ('ukr', 'Українська'),
             ('vie', 'Tiếng Việt'),
             ('wel', 'Cymraeg')
             );

var
  Cfg: TConfig;


procedure SaveConfig();
procedure LoadConfig();


implementation


procedure SaveConfig();
var
  jc: TJSONConfig;
begin
   jc := TJSONConfig.Create(nil);

   jc.Filename := ConfigFilename;
   jc.SetValue('AppWindowName', UnicodeString(Cfg.AppWindowName));
   jc.SetValue('XRes', Cfg.XRes);
   jc.SetValue('YRes', Cfg.YRes);
   jc.SetValue('FullScr', Cfg.FullScr);
   jc.SetValue('FullScrDesk', Cfg.FullScrDesk);
   jc.SetValue('VSync', Cfg.VSync);
   jc.SetValue('Framerate', Cfg.Framerate);
   jc.SetValue('MouseSensitivity', Cfg.MouseSensitivity);
   jc.SetValue('LeftHand', Cfg.LeftHand);
   jc.SetValue('Mute', Cfg.Mute);
   jc.SetValue('VolSound', Cfg.VolSound);
   jc.SetValue('VolMusic', Cfg.VolMusic);
   jc.SetValue('VolSpeech', Cfg.VolSpeech);
   jc.SetValue('Lang', UnicodeString(Cfg.Lang));

   jc.Free();
end;

procedure LoadConfig();
var
  jc: TJSONConfig;
begin
  jc := TJSONConfig.Create(nil);
  jc.Filename := ConfigFilename;

  Cfg.AppWindowName := String(jc.GetValue('AppWindowName', 'Cinetraverse'));
  Cfg.XRes := jc.GetValue('XRes', 1920);
  Cfg.YRes := jc.GetValue('YRes', 1080);
  Cfg.FullScr := jc.GetValue('FullScr', true);
  Cfg.FullScrDesk := jc.GetValue('FullScrDesk', Cfg.FullScr);
  Cfg.VSync := jc.GetValue('VSync', true);
  Cfg.Framerate := jc.GetValue('Framerate', 30);
  Cfg.MouseSensitivity := jc.GetValue('MouseSensitivity', 100);
  Cfg.LeftHand := jc.GetValue('LeftHand', false);
  Cfg.Mute := jc.GetValue('Mute', false);
  Cfg.VolSound := jc.GetValue('VolSound', 100);
  Cfg.VolMusic := jc.GetValue('VolMusic', 100);
  Cfg.VolSpeech := jc.GetValue('VolSpeech', 100);
  Cfg.Lang := String(jc.GetValue('Lang', 'eng'));

  jc.Free();
end;

end.

